#!/usr/bin/perl
#
#   random_gallery.cgi - a CGI program to manage a gallery of images 
#                       and display them at random (or not so random)
#
#       Copyright (C) 2003, Ross A. Beyer, rbeyer@rossbeyer.net
#
#   $Name: random_gallery-1_1 $
#   CVS $Id: random_gallery.cgi,v 1.7 2003/11/30 17:58:42 rbeyer Exp $
#
#
#   License & Copyright Information
#   -------------------------------
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as published
#   by the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   It is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with the User Notes System; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#   or visit their website at www.gnu.org.
#  
#  To learn more about this program, you can read through the POD in
#  this sourcecode, our type "perldoc random_gallery.cgi" at your command
#  line.
#  
#  To add an image to the gallery, read the instructions in the 
#  random_gallery.conf file.
#  

=pod

=head1 NAME

random_gallery.cgi -  a CGI program to manage a gallery of images and display them at random (or not so random)

=head1 SYNOPSIS

This program manages and displays one of a gallery of images.  It
either returns a full page, a snippet of HTML, or just a URL that
can be included into other pages.  There are several different ways
to pick one of your gallery images, and different ways to return
the information.

=head1 DESCRIPTION

This program receives input from an HTML form or query URL and uses
that information to return one of several "gallery" pages, the URL
of the image, or a snippet of HTML that can be included into another
webpage.

To simply add images to the gallery, you can do so by reading the
directions in the file F<random_gallery.conf>, and editing it appropriately.

If you want to edit the kind of HTML that is returned by this
program, you should be able to do so easily.  The writing of the
HTML is accomplished by a number of Subroutines (like
C<print_page_header()>, etc.).  The output is easily customizable
by only altering those subroutines that need it.  By default, this
program outputs HTML written to the XHTML 1.0 strict standard.

There are essentially three modes of output that this program has.
If called with a "return" key, then the output of this program will
be a snippet of HTML (not a full page), intended to be included by
some other page via a server side include.  If the return key is
set to "url", then only the relative URL of an image will be returned
(not HTML).  The other mode is to return a full webpage which is
either a kind of gallery index page, or the particular page of a
single image in the gallery.

=cut

use strict;
use POSIX;
use CGI qw/:standard/;
use File::Basename;

##--- User Alterable Variables ---##

my $random_gallery_conf = "random_gallery.conf";

# This hash has the various potential filename extensions as the
# keys (on the left) and the MIME-types as the values (on the right).
# We need the MIME-types for the return=url option.  Most of the time,
# the MIME-type is identical to the filename extension, but sometimes
# not (as with the .jpg filename, jpeg MIME-type).  Please make sure
# that the keys (the extensions) are lower-case, the rest of the program
# expects them to be that way, although the program can deal with
# image files in the gallery that have upper-case extensions.
my %image_types =   (
                    ".png"  => "png",
                    ".jpg"  => "jpeg",
                    ".jpeg" => "jpeg",
                    ".gif"  => "gif"
                    );

##--- end User Alterable Variables ---##


=pod

=head1 The Gallery

The concept of "the gallery" is simply a list of images in the
directory that the random_gallery.cgi program is in that you want
random_gallery to operate on.  There are two ways for random_gallery
to assemble this list.

The first way is by using a random_gallery.conf configuration file
to list the names of the image files in the directory that should
be on the list.  This file ignores blank lines and lines that start
with a "#", and the images should be one per line.  This mechanism
allows you to explicitly have images in your gallery.  It allows
you to have image files in the directory that are not in your
.conf file that then do not get included.

The second way is by not having a random_gallery.conf file in the
same directory with the random_gallery.cgi file.  If random_gallery.cgi
doesn't find that file, then it will look around the directory for
files that end in well-known image suffixes for the web (.png, .jpg,
.gif, and variations thereof), and it will include all that it
finds.  This allows you to make a directory, put random_gallery.cgi
into, and then just drop images in that directory, and they are
automatically added to the gallery, without having to manage a
configuration file.

=head2 Supporting Gallery Files

There are a couple of other files that can be added to support the
gallery images (if you use a configuration file or not), both a
title file and a caption file, but neither are required.  If they're
there, random_gallery will use them, otherwise it'll just use default
behavior.

=over 4

=item *_title.txt

The title file should contain the title for your image, and should
contain only plain alphanumeric characters.  This file should have
the same root name as your image, with the suffix "_title.txt".
For our example photo.jpg image, the title file would be
"photo_title.txt".

=item *_caption.html

A caption for the image can also be added with a caption file.  The
caption file can contain HTML markup for the caption.  This file
should have the same root name as your image, with the suffix
"_caption.html".  For our example photo.jpg image, the title file
would be "photo_caption.html".

=back

=cut

my @gallery_images;

if( -r "$random_gallery_conf" )
    {
    open( CONFIG, "<$random_gallery_conf")
        or die "The configuration file, $random_gallery_conf, could not be opened.\n";
    
    while (<CONFIG>)
        {
        chomp;                  # no newline
        s/#.*//;                # no comments
        s/^\s+//;               # no leading white
        s/\s+$//;               # no trailing white
        next unless length;     # anything left?
        push @gallery_images, $_;
        } 
    }
else
    {
    my $suffix_pattern = join('|', keys %image_types);

    opendir(DIR, "./") or die "can't opendir ./: $!";
    while( defined(my $file = readdir(DIR)) )
        {
        if( $file =~ /($suffix_pattern)$/gi )
            {
            push @gallery_images, $file;
            }
        }
    closedir(DIR);
    }


#####################################################################
#
# We expect to be passed key/value pairs either from an HTML form
#   with this script as its action, or from a query URL with this
#   script as its href.  In order to parse those incoming key/value
#   pairs, we'll use the CGI module to put them in a hash.


my $cgi_object = CGI->new();
my %form_var;
my $var;
foreach $var ($cgi_object->param())
    {
    $form_var{$var} = $cgi_object->param($var);
    }


#####################################################################
#
#   The main part of this program mostly calls subroutines to acheive 
#   a uniform look over all the different pages that it can present.
#   You can feel free to monkey with this section to arrange the layout
#   how you would like it.

=pod

=head1 Arguments

These are the different arguments that can be passed to the
random_gallery.cgi program either via form variables, or via a
query URL, like so:

    http://somewhere.net//random_gallery.cgi?argument=value

If no arguments are given, the program will return a full HTML
page which will contain the list of the images in its gallery.

=over 4

=item image=photo.jpg

If the only argument to the program is the image argument, then
the program will display a full HTML page for that image that
contains all of the information about that image.

=cut

# Check to see if we should be displaying a gallery page for a
# specific image.
my $number_of_form_var = keys( %form_var );
if( $form_var{"image"} and $number_of_form_var=1 )
    {
    # Check to see if that image is one in our gallery.
    my $found;
    foreach my $image (@gallery_images)
        {
        if ( $image eq $form_var{"image"} )
            {
            $found = 1;
            last;
            }
        }

    if( $found )
        {
        print_image_page( $form_var{"image"} );
        }
    else
        {
        print_image_not_in_gallery( $form_var{"image"} );
        }

    exit(0);
    }



=pod

=item return=*

This argument is the signal to the program that it should return
only a snippet of HTML (no <html>, <head>, <body> or other 
structure tags will be included in the output).  A snippet of
HTML is intended to be included into other web pages with an
include statement of some kind.  It can be done with server side
includes like so:

  <!--#include virtual="/random_gallery.cgi?return=image" -->

The return argument can take many forms, and can take a comma-separated
list of any of the specific return values specified below, although
some exclude others.  They are listed here and there further down.

=cut

if( $form_var{"return"} )
    {
    # This will return only an HTML snippet for inclusion in other
    # pages.  This $snippet variable is what will be printed out.
    my $snippet;

    # The first thing we need to do is figure out which image 
    # to return.
    my $image;

=pod

=item select

The select argument determines how the program will select an image
from the gallery to display.  The select argument is only relevant
when the return argument is specified.  Unlike the return argument,
select can only take one value.

=item select=random

This will randomly select an image from the gallery every time the page
is accessed.  If the select argument is not given, this is the default
behavior.

=cut
    if( $form_var{"select"} eq "random" )
        {
        $image = $gallery_images[ rand @gallery_images ];
        }
=pod

=item select=daily

This will select the same image all day (depending on your computer's notion
of a day), and will rotate through your images in an orderly fashion, one
per day.

=cut
    elsif( $form_var{"select"} eq "daily" )
        {
        # This is a crude estimate of the number of days since
        # the epoch, January 1, 1970.  It would have been slicker
        # to use the Date::Calc module, but that would be one more
        # thing for this program to require.
        my $delta_days = int( time()/86400 );

        my $image_count = @gallery_images;
    
        my $index = $delta_days % $image_count;

        $image = $gallery_images[$index];
        }
=pod

=item select=photo.jpg

If the user specified both a return argument and the image argument
with the name of an image in our gallery, the program will always return
that particular image.  Although if a user always wanted to display 
the same image, they could just use the <img> tag directly, and
using this program is overkill, but someone might want it, so it
is provided.

=cut
    elsif( $form_var{"select"} )
        {
        my $found;
        foreach my $im (@gallery_images)
            {
            if ( $im eq $form_var{"select"} )
                {
                $found = 1;
                last;
                }
            }

        if( $found )
            {
            $image = $form_var{"select"};
            }
        else
            {
            print header();
            print qq(The value, $form_var{"select"}, is either not an image in the gallery, or isn't a known value for the select argument.);
            exit(0);
            }
        }
    else    
        {
        # I know that this replicates code, but I couldn't think of another
        # way to solve this logic for the select argument.
        $image = $gallery_images[ rand @gallery_images ];
        }

    # We now have a value for $image.  Now we need to get some
    # other info so we can fill out the img tag.
    
    # Get the URL from the SCRIPT_NAME environment variable.
    my $url = $ENV{"SCRIPT_NAME"};
    my ($base, $dir, $ext) = fileparse($url);
    my $src = $dir.$image;

    my $gallery_link = qq(<a href="$url?image=$image">);

=pod

=item return=url

The specification of "url" to the key "return" will cause the program
to act as if it was an image URL, and the data that comes back out
of the program will be image data.  It will not return any HTML.
So anywhere that you would use an image URL you could use this.
You could use random_gallery as follows in an HTML file:

  <img src="/random_gallery.cgi?return=url" />

or it could be used when you need to specify a URL for cascading
style sheets:

  div.random
    {
    background-image: url(/random_gallery.cgi?return=url);
    }

=cut

    if( $form_var{"return"} eq "url" )
        {
        # Figure out which MIME-type to send
        my ($base, $dir, $ext) = fileparse($image, '\..*');
        my $lc_ext = lc( $ext );

        print "Content-Type: image/$image_types{$lc_ext}\n\n";

        binmode(STDOUT);
        open (FILE, "$image") || exit;
        binmode(FILE);
        my $chunk;
        while( my $bytesread=read(FILE,$chunk,65536) )
            {
            print "$chunk";
            }
        close(FILE);

        exit(0);
        }

=pod

=item class=something

You are using cascading style sheets (CSS), right?  Well you'll
probably want to be able to specify a class for the <img> tag that
gets returned in the snippet, and the other possible tags.  So the
class argument is a way to do that.  If you specify the class
argument, its value will be added to the tags of B<all> of the
elements this program returns.

This may be a little heavy handed, the alternative would be to
try and specify classes for all of the individual tags that get
returned in the snippet, which seems a little complicated.
Hopefully this will serve your CSS needs.

=cut
    my $class;
    if( $form_var{"class"} )
        {
        $class = qq( class="$form_var{"class"}");
        }

=pod

=item caption_tag=<span|div>

I couldn't decide if the caption should be returned enclosed in a 
<span> or a <div> tag, so I decided to let you specify, if you want.
If you don't, the default is caption=span.

=cut
    my $caption_tag;
    if( $form_var{"caption_tag"} eq "div" )
        {
        $caption_tag = "div";
        }
    else
        {
        $caption_tag = "span";
        }


    # Now we have to parse the "return" variable for all of the different
    # things that it might have.

    my %return;
    my @returns = split( /,/,$form_var{"return"} );
    foreach my $returned (@returns)
        {
        $return{$returned} = $returned;
        }   


    # Get the title
    my $ref_image_titles = get_titles( $image );
    my $title = $$ref_image_titles{$image};


    # The POD for this is below for layout reasons.
    if( $return{"image"} )
        {
        my $img = qq(<img src="$src"$class alt="$title" title="$title" />);

=pod

=item gallery_link=<image|caption|image,caption>

The gallery_link argument indicates what to wrap an anchor tag 
(<a href>) around that points to the page returned by using the 
image=something.jpg argument for the image selected.  The value of
gallery_link can be either image, caption, or both image and caption
separated by a comma.

You many not want to use the "caption" value if you have more complicated
HTML in your caption.

=cut
# This is not technically true.  At the moment, I'm just regexing
# the value of gallery_link for the image and caption strings, but
# I may want to do something more complicated in the future.
    
        if( $form_var{"gallery_link"} =~ /image/ )
            {
            $img = qq($gallery_link$img</a>);
            }

        $snippet .= $img;
        }

=pod

=item return=image

This return value indicates that an <img> tag with an image specified
by the select argument should be returned in the snippet.  Odds are 
good you'll always be using the image value.  I know that it might
seem crazy to allow people to not get an <img> tag (by forgetting to
use the image value to return) out of a program that is supposed to
display images, but someone might want it, so there you go.

=item return=caption

This value to the return argument will cause the caption for the specified
image to be appended to the returned HTML snippet.  If that image has no
caption, nothing will be added.

=item return=caption_1st

This is similar to the above option, but returns the caption before
the <img> tag in the snippet, rather that afterwards.

=cut
    if( $return{"caption"} or $return{"caption_1st"} )
        {
        my $ref_image_captions = get_captions( $image );
        my $caption = $$ref_image_captions{$image};
        if( $caption )
            {
            $caption = "<$caption_tag$class>$caption</$caption_tag>";
            if( $form_var{"gallery_link"} =~ /caption/ )
                {
                $caption = qq($gallery_link$caption</a>);
                }

            if( $return{"caption_1st"} )
                {
                $snippet =  "$caption$snippet";
                }
            else
                {
                $snippet .= $caption;
                }
            }
        }

    unless( $snippet )
        {
        $snippet = 
        "<!-- The value for the return argument is not recognized. -->";
        }

    print header(); 
    print $snippet;
    exit(0);
    }


# If we didn't get any special variables, we simply present a kind
# of index page for our gallery.

print_page_header();

print_index_page( @gallery_images );

print_page_footer();

exit(0);


######################################################################
#
# Subroutines
#
######################################################################

# Print out top-of-page stuff.  Replace with your own HTML.
sub print_page_header
    {
    my $incoming_title = shift;
    my $title = "Image Gallery";
    if( $incoming_title )
        {
        $title = $incoming_title;
        }

    # If you are doing any SSI in these files, make sure that 
    # "AddOutputFilter INCLUDES .cgi" set for these files, if you
    # are using Apache2.

    # Use the CGI module's method to send the Content-type line
    print header();

    print qq(<!DOCTYPE html
    PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<title>$title</title>
</head>

<body>
);
    }

# Print out bottom-of-page stuff. Replace with your own HTML.
sub print_page_footer
    {
    print qq(

<p>
<small>
Gallery managed by 
<a href="http://RossBeyer.net/software/random_gallery">random_gallery</a>.
</small>
</p>

</body>
</html>);
    }

# This prints out the default page body.
sub print_index_page
    {
    my @gallery_images = @_;
    my $url = url();

    print qq(<h1>Image Gallery</h1>

<p>
These are the images in this gallery:
</p>

<ul>
);

    my $ref_image_titles = get_titles( @gallery_images );

    foreach my $image (@gallery_images)
        {
        print 
        qq(<li><a href="$url?image=$image">$$ref_image_titles{$image}</a></li>\n);
        }

    print qq(</ul>\n);
    }


# Prints out the page with all the information we can get on the image.
sub print_image_page
    {
    my $image = pop;
    my $url = url();

    my $title;
    my $caption;

    my $ref_image_titles = get_titles( $image );
    $title = $$ref_image_titles{$image};

    my $ref_image_captions = get_captions( $image );
    $caption = $$ref_image_captions{$image};

    print_page_header( $title );

    print qq(<h1>$title</h1>

<img src="$image" alt="$title" title="$title" />

);

    if( $caption )
        {
        print qq(
<p>
$caption
</p>

);
        }

    print_page_footer();
    }


# This prints out the page indicating we didn't find the image in our
# gallery.
sub print_image_not_in_gallery
    {
    my $image_not_found = pop;
    my $url = url();

    print_page_header( "$image_not_found not found" );

    print qq(<h1>Image $image_not_found not found</h1>

<p>
This image could not be found in the <a href="$url">gallery</a>.
</p>

);

    print_page_footer();
    }

# This gets the title information for the image(s) requested or just
# defaults to the image file name.
sub get_titles
    {
    my @images = @_;

    my %titles;

    foreach my $image (@images)
        {
        my ($base, $dir, $ext) = fileparse($image, '\..*');
        my $title_file = $base."_title.txt";
        if( open(TITLE, "<$title_file") )
            {
            my $title = <TITLE>;
            close( TITLE );
            chomp( $title );
            $titles{$image} = $title;
            }
        else
            {
            $titles{$image} = $image;
            }
        }
    
    return( \%titles );
    }

# This gets the caption information for the image(s) requested or just
# defaults to blank.
sub get_captions
    {
    my @images = @_;

    my %captions;

    foreach my $image (@images)
        {
        my ($base, $dir, $ext) = fileparse($image, '\..*');
        my $caption_file = $base."_caption.html";
        if( open(CAPTION, "<$caption_file") )
            {
            my @caption = <CAPTION>;
            close( CAPTION );
            $captions{$image} = join("", @caption);
            }
        else
            {
            $captions{$image} = "";
            }
        }
    
    return( \%captions );
    }




=pod

=back

=head1 Examples

Using the return argument will only return a snippet of HTML, so you
should use some handy mechanism like server side includes to include
the output of this program into other web pages.

This returns only an <img> tag in the snippet, and uses the default
random behavior to select that image.

    random_gallery.cgi?return=image

This returns only the image from a specific image in the gallery (sun.jpg
in this case):

    random_gallery.cgi?return=image&select=sun.jpg

This returns the image, wrapped in a link back to its full page, and 
the caption:

    random_gallery.cgi?return=image,caption&gallery_link=image

For other examples, see the random_gallery website at
http://RossBeyer.net/software/random_gallery


=head1 AUTHOR

Ross A. Beyer, rbeyer@RossBeyer.net

=cut
